# -*- coding: utf-8 -*-

from main import *
from Creeps import *
from Projectiles import *
import widgets
from utils import get_Percentage

class Tower(Sprite):
    """A tower object, placed on the grid. Unwalkable. Fires projectiles at nearby creeps."""
    def __init__(self, screen, game, position=(0, 0), tower_image='images/towers/arrow_tower_1.png', projectile_image='images/projectiles/Arrow1.png', template=False):
        """ Create a new Tower.
        screen:
        The screen on which the tower is located (must be a pygame Surface object, such as pygame.display)
        game:
        This is the game object that holds information about the game world.
        tower_image:
        An image (as Pygame surface objects) for the tower.
        projectile_image:
        An image for the projectile the tower emits
        position:
        The (x, y) position of the tower.
        """
        Sprite.__init__(self)

        self.name = "Arrow Tower"

        if not template:
            game._placed_tower_count += 1
            self.id = game._placed_tower_count
            game.last_placed_tower_id = self.id
        else:
            self.id = None

        self.type = 'Tower'
        self.screen = screen
        self.game = game
        self.image = pygame.image.load(tower_image).convert_alpha()
        self.icon = pygame.transform.scale(self.image, (20, 20))
        self.projectile_image = pygame.image.load(projectile_image).convert_alpha()
        self.orig_pos = position  #just for eventual need to know exactly which pixel it was placed at (and then it was snapped to the topleft of that tile)

        if not template:
            #Tile snapping code
            self.coordx_topleft, self.coordy_topleft = self.game.xy2coord(position) #these are used further down
            self.coord_topleft = (self.coordx_topleft, self.coordy_topleft)
            self.pos = game.coord2xy(self.coord_topleft)


        self.cost = 10

        #Fire-related
        self.damage = 4
        self.radius = 100 #Attacking range
        self.ratio = 1.0 #Ratio of the radius, used for upgrades. #is this really used by current collison detection?
        self.attack_speed = 700 #ms delay
        self.last_fire_time = 0
        self.last_target_id = None

        #Experience-related
        self.level = 1
        self.experience = 0
        self.kills = 0
        self.experience_req = 5
        self.last_experience_req = 0
        self.delta_exp_req = self.experience_req - self.last_experience_req

        #Projectile-related
        self.projectile_speed = 0.18


        self.width, self.height = self.image.get_size()
        self.widthbycoord = ceil(self.width / float(game.GRID_SIZE)) #get the ceiling, to block all tiles the tower occupies a portion of.
        self.heightbycoord = ceil(self.height / float(game.GRID_SIZE))

        #Blocking all covered tiles
        #
        if not template:
            for extra_tile_y in range(int(self.heightbycoord)): #Block all occupied tiles
                for extra_tile_x in range(int(self.widthbycoord)):
                    self.game.gridpath.set_blocked((self.coordx_topleft + extra_tile_x, self.coordy_topleft + extra_tile_y))

            self.rect = Rect(self.pos[0], self.pos[1], self.width, self.height)
        #print("Image: ", tower_image, "projectile_image: ", projectile_image, "Position xy: ", self.pos, "Position Coord: ", self.coord_topleft)

    def draw(self): ## If this turns out to be the better way, don't forget to change the drawing method from self.towers.draw(self.screen) to for tower in self.towers: tower.draw()
        self.screen.blit(self.image, self.rect)

        # The experience bar is 15x4 px.
        #
        experience_bar_length = 15
        experience_bar_height = 4

        experience_percentage = get_Percentage((self.experience_req - self.last_experience_req), (self.experience - self.last_experience_req)) #float fraction, like 0.333333 (equals 33.3333% )

        experience_bar_fill_length = int(ceil(experience_percentage * experience_bar_length))
        experience_bar_x = self.rect.centerx - (experience_bar_length / 2)
        experience_bar_y = self.rect.centery - 10
        self.screen.fill(   Color('grey'),
        (experience_bar_x, experience_bar_y, experience_bar_length, experience_bar_height))
        self.screen.fill(   Color('blue'),
        (   experience_bar_x, experience_bar_y,
        experience_bar_fill_length, experience_bar_height))


    def fire(self, targets):
        if self.last_fire_time + self.attack_speed < self.game.total_time_passed:
            last_target = self.game.lookup_creep(self.last_target_id)
            if self.last_target_id in targets and last_target.state == 0: #ALIVE state
                target_id = self.last_target_id
            else:
                target_id = choice(targets)

            self.projectile = Projectile(self.screen, self.game, self, self.game.lookup_creep(target_id))


            self.game.attacks += 1
            self.last_fire_time = self.game.total_time_passed
            self.last_target_id = target_id

    def sell(self):
        sellamount = int(round(0.75 * self.cost))
        self.game.money += sellamount
        self.game.text_messages.append(widgets.TextMessage(self.game.screen, 'Sold! '+'+'+str(sellamount), vec2d(self.pos[0], self.pos[1]-22), duration=3100, size=23, initialdelay=800))
        self.game.selection_info_active = False
        for extra_tile_y in range(int(self.heightbycoord)): #Block all occupied tiles
            for extra_tile_x in range(int(self.widthbycoord)):
                self.game.gridpath.set_blocked((self.coordx_topleft + extra_tile_x, self.coordy_topleft + extra_tile_y), blocked=False)
        self.kill()

    def add_experience(self, amount):
        self.kills +=1
        self.experience += amount
        if self.experience >= self.experience_req:
            self.level_up()

    def level_up(self):
        self.level += 1
        self.last_experience_req = self.experience_req
        self.experience_req += 5 * (self.level * self.level)
        self.delta_exp_req = self.experience_req - self.last_experience_req
        self.damage += 2
        if self.experience >= self.experience_req:
            print "I leveled up another time!"
            self.level_up()

class Tower_1(Tower):
    def __init__(self, screen, game, position=(0, 0), tower_image='images/towers/arrow_tower_1.png', projectile_image='images/projectiles/Arrow1.png', template=False): #tower_image='images/towers/arrow_tower_1.png', projectile_image='images/projectiles/Arrow1.png', template=False):
        Tower.__init__(self, screen, game, position, tower_image, projectile_image, template)
        self.name = "Basic " + self.name
        self.damage = 14
        self.attack_speed = 900

        self.cost = 10
    def level_up(self):
        Tower.level_up(self)
        self.damage += 3

class Tower_2(Tower):
    def __init__(self, screen, game, position=(0, 0), tower_image='images/towers/arrow_tower_2.png', projectile_image='images/projectiles/Arrow2.png', template=False):
        Tower.__init__(self, screen, game, position, tower_image, projectile_image, template)
        self.name = "Improved " + self.name
        self.damage = 10
        self.attack_speed = 300
        self.radius = 150
        self.cost = 25

class Tower_3(Tower):
    def __init__(self, screen, game, position=(0, 0), tower_image='images/towers/frost_tower.png', projectile_image='images/projectiles/Frost_arrow.png', template=False):
        Tower.__init__(self, screen, game, position, tower_image, projectile_image, template)
        self.name = "Ice " + self.name
        self.damage = 28
        self.attack_speed = 500
        self.radius = 110
        self.cost = 70
class Tower_4(Tower):
    def __init__(self, screen, game, position=(0, 0), tower_image='images/towers/Firetower.png', projectile_image='images/projectiles/Fire_arrow.png', template=False):
        Tower.__init__(self, screen, game, position, tower_image, projectile_image, template)
        self.name = "Fire " + self.name
        self.damage = 38
        self.attack_speed = 400
        self.radius = 100
        self.cost = 120
